#ifdef MOZ_TREE_PIXMAN
#define pixman_composite_glyphs _moz_pixman_composite_glyphs
#define pixman_composite_glyphs_no_mask _moz_pixman_composite_glyphs_no_mask
#define pixman_glyph_cache_create _moz_pixman_glyph_cache_create
#define pixman_glyph_cache_destroy _moz_pixman_glyph_cache_destroy
#define pixman_glyph_cache_freeze _moz_pixman_glyph_cache_freeze
#define pixman_glyph_cache_insert _moz_pixman_glyph_cache_insert
#define pixman_glyph_cache_lookup _moz_pixman_glyph_cache_lookup
#define pixman_glyph_cache_remove _moz_pixman_glyph_cache_remove
#define pixman_glyph_cache_thaw _moz_pixman_glyph_cache_thaw
#define pixman_glyph_get_extents _moz_pixman_glyph_get_extents
#define pixman_glyph_get_mask_format _moz_pixman_glyph_get_mask_format
#define pixman_region_set_static_pointers _moz_pixman_region_set_static_pointers
#define pixman_region_init _moz_pixman_region_init
#define pixman_region_init_rect _moz_pixman_region_init_rect
#define pixman_region_init_rects _moz_pixman_region_init_rects
#define pixman_region_init_with_extents _moz_pixman_region_init_with_extents
#define pixman_region_fini _moz_pixman_region_fini
#define pixman_region_translate _moz_pixman_region_translate
#define pixman_region_copy _moz_pixman_region_copy
#define pixman_region_intersect _moz_pixman_region_intersect
#define pixman_region_union _moz_pixman_region_union
#define pixman_region_union_rect _moz_pixman_region_union_rect
#define pixman_region_subtract _moz_pixman_region_subtract
#define pixman_region_inverse _moz_pixman_region_inverse
#define pixman_region_contains_point _moz_pixman_region_contains_point
#define pixman_region_contains_rectangle _moz_pixman_region_contains_rectangle
#define pixman_region_not_empty _moz_pixman_region_not_empty
#define pixman_region_extents _moz_pixman_region_extents
#define pixman_region_n_rects _moz_pixman_region_n_rects
#define pixman_region_rectangles _moz_pixman_region_rectangles
#define pixman_region_equal _moz_pixman_region_equal
#define pixman_region_selfcheck _moz_pixman_region_selfcheck
#define pixman_region_reset _moz_pixman_region_reset
#define pixman_region_clear _moz_pixman_region_clear
#define pixman_region_print _moz_pixman_region_print
#define pixman_region32_init _moz_pixman_region32_init
#define pixman_region32_init_rect _moz_pixman_region32_init_rect
#define pixman_region32_init_rects _moz_pixman_region32_init_rects
#define pixman_region32_init_with_extents _moz_pixman_region32_init_with_extents
#define pixman_region32_init_from_image _moz_pixman_region32_init_from_image
#define pixman_region32_fini _moz_pixman_region32_fini
#define pixman_region32_translate _moz_pixman_region32_translate
#define pixman_region32_copy _moz_pixman_region32_copy
#define pixman_region32_intersect _moz_pixman_region32_intersect
#define pixman_region32_intersect_rect _moz_pixman_region32_intersect_rect
#define pixman_region32_union _moz_pixman_region32_union
#define pixman_region32_union_rect _moz_pixman_region32_union_rect
#define pixman_region32_subtract _moz_pixman_region32_subtract
#define pixman_region32_inverse _moz_pixman_region32_inverse
#define pixman_region32_contains_point _moz_pixman_region32_contains_point
#define pixman_region32_contains_rectangle _moz_pixman_region32_contains_rectangle
#define pixman_region32_not_empty _moz_pixman_region32_not_empty
#define pixman_region32_extents _moz_pixman_region32_extents
#define pixman_region32_n_rects _moz_pixman_region32_n_rects
#define pixman_region32_rectangles _moz_pixman_region32_rectangles
#define pixman_region32_equal _moz_pixman_region32_equal
#define pixman_region32_selfcheck _moz_pixman_region32_selfcheck
#define pixman_region32_reset _moz_pixman_region32_reset
#define pixman_region32_clear _moz_pixman_region32_clear
#define pixman_region32_print _moz_pixman_region32_print
#define pixman_blt _moz_pixman_blt
#define pixman_fill _moz_pixman_fill
#define pixman_transform_point_3d _moz_pixman_transform_point_3d
#define pixman_version _moz_pixman_version
#define pixman_version_string _moz_pixman_version_string
#define pixman_format_supported_destination _moz_pixman_format_supported_destination
#define pixman_format_supported_source _moz_pixman_format_supported_source
#define pixman_image_create_solid_fill _moz_pixman_image_create_solid_fill
#define pixman_image_create_linear_gradient _moz_pixman_image_create_linear_gradient
#define pixman_image_create_radial_gradient _moz_pixman_image_create_radial_gradient
#define pixman_image_create_conical_gradient _moz_pixman_image_create_conical_gradient
#define pixman_image_create_bits _moz_pixman_image_create_bits
#define pixman_image_ref _moz_pixman_image_ref
#define pixman_image_unref _moz_pixman_image_unref
#define pixman_image_set_clip_region _moz_pixman_image_set_clip_region
#define pixman_image_set_clip_region32 _moz_pixman_image_set_clip_region32
#define pixman_image_set_has_client_clip _moz_pixman_image_set_has_client_clip
#define pixman_image_set_transform _moz_pixman_image_set_transform
#define pixman_image_set_repeat _moz_pixman_image_set_repeat
#define pixman_image_set_filter _moz_pixman_image_set_filter
#define pixman_image_set_source_clipping _moz_pixman_image_set_source_clipping
#define pixman_image_set_alpha_map _moz_pixman_image_set_alpha_map
#define pixman_image_set_component_alpha _moz_pixman_image_set_component_alpha
#define pixman_image_set_accessors	 _moz_pixman_image_set_accessors	
#define pixman_image_set_indexed	 _moz_pixman_image_set_indexed	
#define pixman_image_get_data _moz_pixman_image_get_data
#define pixman_image_get_width _moz_pixman_image_get_width
#define pixman_image_get_height _moz_pixman_image_get_height
#define pixman_image_get_stride _moz_pixman_image_get_stride
#define pixman_image_get_depth _moz_pixman_image_get_depth
#define pixman_image_fill_rectangles	 _moz_pixman_image_fill_rectangles	
#define pixman_compute_composite_region _moz_pixman_compute_composite_region
#define pixman_image_composite _moz_pixman_image_composite
#define pixman_sample_ceil_y _moz_pixman_sample_ceil_y
#define pixman_sample_floor_y _moz_pixman_sample_floor_y
#define pixman_edge_step _moz_pixman_edge_step
#define pixman_edge_init _moz_pixman_edge_init
#define pixman_line_fixed_edge_init _moz_pixman_line_fixed_edge_init
#define pixman_rasterize_edges _moz_pixman_rasterize_edges
#define pixman_add_traps _moz_pixman_add_traps
#define pixman_add_trapezoids _moz_pixman_add_trapezoids
#define pixman_add_triangles _moz_pixman_add_triangles
#define pixman_composite_trapezoids _moz_pixman_composite_trapezoids
#define pixman_composite_triangles _moz_pixman_composite_triangles
#define pixman_rasterize_trapezoid _moz_pixman_rasterize_trapezoid
#define pixman_disable_out_of_bounds_workaround _moz_pixman_disable_out_of_bounds_workaround
#define pixman_f_transform_bounds _moz_pixman_f_transform_bounds
#define pixman_f_transform_from_pixman_transform _moz_pixman_f_transform_from_pixman_transform
#define pixman_f_transform_init_identity _moz_pixman_f_transform_init_identity
#define pixman_f_transform_init_rotate _moz_pixman_f_transform_init_rotate
#define pixman_f_transform_init_scale _moz_pixman_f_transform_init_scale
#define pixman_f_transform_init_translate _moz_pixman_f_transform_init_translate
#define pixman_f_transform_invert _moz_pixman_f_transform_invert
#define pixman_f_transform_multiply _moz_pixman_f_transform_multiply
#define pixman_f_transform_point _moz_pixman_f_transform_point
#define pixman_f_transform_point_3d _moz_pixman_f_transform_point_3d
#define pixman_f_transform_rotate _moz_pixman_f_transform_rotate
#define pixman_f_transform_scale _moz_pixman_f_transform_scale
#define pixman_f_transform_translate _moz_pixman_f_transform_translate
#define pixman_image_composite32 _moz_pixman_image_composite32
#define pixman_image_fill_boxes _moz_pixman_image_fill_boxes
#define pixman_image_get_component_alpha _moz_pixman_image_get_component_alpha
#define pixman_image_get_destroy_data _moz_pixman_image_get_destroy_data
#define pixman_image_get_format _moz_pixman_image_get_format
#define pixman_image_set_destroy_function _moz_pixman_image_set_destroy_function
#define pixman_region_init_from_image _moz_pixman_region_init_from_image
#define pixman_region_intersect_rect _moz_pixman_region_intersect_rect
#define pixman_transform_bounds _moz_pixman_transform_bounds
#define pixman_transform_from_pixman_f_transform _moz_pixman_transform_from_pixman_f_transform
#define pixman_transform_init_identity _moz_pixman_transform_init_identity
#define pixman_transform_init_rotate _moz_pixman_transform_init_rotate
#define pixman_transform_init_scale _moz_pixman_transform_init_scale
#define pixman_transform_init_translate _moz_pixman_transform_init_translate
#define pixman_transform_invert _moz_pixman_transform_invert
#define pixman_transform_is_identity _moz_pixman_transform_is_identity
#define pixman_transform_is_int_translate _moz_pixman_transform_is_int_translate
#define pixman_transform_is_inverse _moz_pixman_transform_is_inverse
#define pixman_transform_is_scale _moz_pixman_transform_is_scale
#define pixman_transform_multiply _moz_pixman_transform_multiply
#define pixman_transform_point _moz_pixman_transform_point
#define pixman_transform_rotate _moz_pixman_transform_rotate
#define pixman_transform_scale _moz_pixman_transform_scale
#define pixman_transform_translate _moz_pixman_transform_translate
#endif
