#define MOZ_SOURCE_STAMP 6f82d30fe05e1412e744cb76af86f0c9ffe509d4
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/6f82d30fe05e1412e744cb76af86f0c9ffe509d4
