/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(c3971bd9-0fbf-48d3-9498-0ac340d0d216)]
interface nsIPaymentProviderStrategy : nsISupports
{
  attribute DOMString paymentServiceId;
  readonly attribute jsval iccInfo;

  void cleanup();
};
